import click
import json

from . import __version__
from .api import MouserOrderRequest, MouserPartSearchRequest
from .api import MouserCartRequest, MouserOrderHistoryRequest


# API Keys File (Default)
API_KEYS_FILE = 'mouser_api_keys.yaml'


@click.command()
@click.version_option(version=__version__, prog_name='Mouser Python API')
@click.argument('request_type', type=click.Choice(['cart', 'history', 'order', 'search'], case_sensitive=False), required=True)
@click.argument('operation', required=True)
@click.option('--number', required=False, help='Part or Order number.')
@click.option('--keyword', required=False, help='Keyword(s) to search')
@click.option('--record-limit', required=False, help='How many records --keyword search should return')
@click.option('--option', required=False, help='Options supported by Mouser search engine. Available options depend on OPERATION.')
@click.option('--export', is_flag=True, required=False, help='Export data to CSV.')
def mouser_cli(request_type, operation, number, keyword, record_limit, option, export):
    ''' Main CLI entry point '''

    args = []

    # Create request
    if request_type == 'order':
        args.append(number)
        request = MouserOrderRequest(operation, API_KEYS_FILE, *args)

        if request.url:
            print(f'[LINK]\t{request.api_url}')
            # Run request
            request.run()

            if export:
                # Export order lines to CSV
                filename = request.export_order_lines_to_csv(order_number=number, clean=True)
                print(f'[DATA]\tOrder lines exported to {filename}')
            else:
                # Print it
                print('[DATA]')
                request.print_response()

    elif request_type == 'search':
        request = MouserPartSearchRequest(operation, API_KEYS_FILE, *args)

        if request.url:
            print(f'[LINK]\t{request.api_url}')

            if operation == 'partnumber':
                if not number:
                    print('[ERROR]\tMissing Mouser Part Number')
                else:
                    if option not in [None, 'None', 'Exact']:
                        print('[ERROR]\t --option must be [None|Exact]')
                    else:
                        # Run request
                        search = request.part_search(number, option)
                        # Print body
                        print('[BODY]')
                        print(json.dumps(request.body, indent=4, sort_keys=True))
                        if search:
                            # Print result
                            print('[DATA]')
                            request.print_clean_response()

            if operation == 'keyword':
                if not keyword:
                    print('[ERROR]\t Missing --keyword flag and keyword(s)')
                else:
                    if option not in [None, 'None', 'Rohs', 'InStock', 'RohsAndInStock']:
                        print('[ERROR]\t --option must be [None|Rohs|InStock|RohsAndInStock]')
                    else:
                        # Run request
                        search = request.keyword_search(keyword, record_limit, option)
                        # Print body
                        print('[BODY]')
                        print(json.dumps(request.body, indent=4, sort_keys=True))
                        if search:
                            # Print result
                            print('[DATA]')
                            request.print_clean_response()

    elif request_type == 'cart':
        request = MouserCartRequest(operation, API_KEYS_FILE, *args)

    elif request_type == 'history':
        request = MouserOrderHistoryRequest(operation, API_KEYS_FILE, *args)



import json

from .base import MouserBaseRequest


class MouserCartRequest(MouserBaseRequest):
    """ Mouser Cart Request """

    name = 'Cart'
    operations = {
        'get': ('', ''),
        'update': ('', ''),
        'insertitem': ('', ''),
        'updateitem': ('', ''),
        'removeitem': ('', ''),
    }


class MouserOrderHistoryRequest(MouserBaseRequest):
    """ Mouser Order History Request """

    name = 'Order History'
    operations = {
        'ByDateFilter': ('', ''),
        'ByDateRange': ('', ''),
    }


class MouserOrderRequest(MouserBaseRequest):
    """ Mouser Order Request """

    name = 'Order'
    operations = {
        'get': ('GET', '/order'),
        'create': ('', ''),
        'submit': ('', ''),
        'options': ('', ''),
        'currencies': ('', ''),
        'countries': ('', ''),
    }

    def export_order_lines_to_csv(self, order_number='', clean=False):
        ''' Export Order Lines to CSV '''

        def convert_order_lines_to_list(clean=False):

            if clean:
                # Exclude following columns
                exclude_col = [
                    'Errors',
                    'MouserATS',
                    'PartsPerReel',
                    'ScheduledReleases',
                    'InfoMessages',
                    'CartItemCustPartNumber',
                    'LifeCycle',
                    'SalesMultipleQty',
                    'SalesMinimumOrderQty',
                    'SalesMaximumOrderQty',
                ]
            else:
                exclude_col = []

            response_data = self.get_response()
            data_list = []
            if 'OrderLines' in response_data:
                order_lines = response_data['OrderLines']

                headers = [key for key in order_lines[0] if key not in exclude_col]
                data_list.append(headers)

                for order_line in order_lines:
                    line = [value for key, value in order_line.items() if key not in exclude_col]
                    data_list.append(line)

                return data_list

        data_to_export = convert_order_lines_to_list(clean)
        filename = '_'.join([self.name, order_number]) + '.csv'
        # Export to CSV file
        self.export_csv(filename, data_to_export)

        return filename


class MouserPartSearchRequest(MouserBaseRequest):
    """ Mouser Part Search Request """

    name = 'Part Search'
    operations = {
        'keyword': ('POST', '/search/keyword'),
        'keywordandmanufacturer': ('', ''),
        'partnumber': ('POST', '/search/partnumber'),
        'partnumberandmanufacturer': ('', ''),
        'manufacturerlist': ('', ''),
    }

    def get_clean_response(self):
        cleaned_part_data = {
            'Availability': '',
            'Category': '',
            'DataSheetUrl': '',
            'Description': '',
            'ImagePath': '',
            'Manufacturer': '',
            'ManufacturerPartNumber': '',
            'MouserPartNumber': '',
            'ProductDetailUrl': '',
            'ProductAttributes': [],
            'PriceBreaks': [],
        }

        response = self.get_response()
        if self.get_response():
            try:
                parts = response['SearchResults'].get('Parts', [])
            except AttributeError:
                parts = None

            if parts:
                cleaned_data = []
                
                if self.operation == 'partnumber':
                    # Process first part
                    first_part = parts[0]
                    # Merge
                    cleaned_part = {}
                    for key in cleaned_part_data:
                        cleaned_part[key] = first_part.get(key, None)
                    cleaned_data.append(cleaned_part)

                if self.operation == 'keyword':

                    # Process all parts
                    for part in parts:
                        cleaned_part = {}
                        for key in cleaned_part_data:
                            cleaned_part[key] = part.get(key, None)
                        cleaned_data.append(cleaned_part)

                return cleaned_data

    def print_clean_response(self):
        response_data = self.get_clean_response()
        print(json.dumps(response_data, indent=4, sort_keys=True))

    def get_body(self, **kwargs):

        body = {}

        if self.operation == 'partnumber':
            part_number = kwargs.get('part_number', None)
            option = kwargs.get('option', 'None')

            if part_number:
                body = {
                    'SearchByPartRequest': {
                        'mouserPartNumber': part_number,
                        'partSearchOptions': option,
                    }
                }
        if self.operation == 'keyword':
            keyword = kwargs.get('keyword', None)
            record_limit = kwargs.get('record_limit', None)
            option = kwargs.get('option', 'None')

            if keyword:
                body = {
                    'SearchByKeywordRequest': {
                        "keyword": keyword,
                        "records": record_limit,
                        "startingRecord": 0,
                        "searchOptions": option,
                        "searchWithYourSignUpLanguage": "en",
                    }
                }

        return body

    def part_search(self, part_number, option='None'):
        '''Mouser Part Number Search '''

        kwargs = {
            'part_number': part_number,
            'option': option,
        }

        self.body = self.get_body(**kwargs)

        if self.api_key:
            return self.run(self.body)
        else:
            return False
        
    def keyword_search(self, keyword, record_limit=0, option='None'):
        '''Mouser Keyword Search'''

        kwargs = {
            'keyword': keyword,
            'record_limit': record_limit,
            'option': option,
        }

        self.body = self.get_body(**kwargs)

        if self.api_key:
            return self.run(self.body)
        else:
            return False



import os
import csv
import json
import requests


# Mouser Base URL
BASE_URL = 'https://api.mouser.com/api/v1.0'


def get_api_keys(filename=None):
    """ Mouser API Keys """

    # Look for API keys in environmental variables
    api_keys = [
        os.environ.get('MOUSER_ORDER_API_KEY', ''),
        os.environ.get('MOUSER_PART_API_KEY', ''),
    ]

    # Else look into configuration file
    if not (api_keys[0] or api_keys[1]) and filename:
        try:
            with open(filename, 'r') as keys_in_file:
                api_keys = []

                for key in keys_in_file:
                    api_keys.append(key.replace('\n', ''))

                if len(api_keys) == 2:
                    return api_keys
                else:
                    pass
        except FileNotFoundError:
            print(f'[ERROR]\tAPI Keys File "{filename}" Not Found')

    return api_keys


class MouserAPIRequest:
    """ Mouser API Request """

    url = None
    api_url = None
    method = None
    body = {}
    response = None
    api_key = None

    def __init__(self, url, method, file_keys=None, *args):
        if not url or not method:
            return None
        self.api_url = BASE_URL + url
        self.method = method

        # Append argument
        if len(args) == 1:
            self.api_url += '/' + str(args[0])

        # Append API Key
        if self.name == 'Part Search':
            self.api_key = get_api_keys(file_keys)[1]
        else:
            self.api_key = get_api_keys(file_keys)[0]

        if self.api_key:
            self.url = self.api_url + '?apiKey=' + self.api_key
        else:
            raise FileNotFoundError('API Keys Are Missing')

    def get(self, url):
        response = requests.get(url=url)
        return response

    def post(self, url, body):
        headers = {
            'Content-Type': 'application/json',
        }
        response = requests.post(url=url, data=json.dumps(body), headers=headers)
        return response

    def run(self, body={}):
        if self.method == 'GET':
            self.response = self.get(self.url)
        elif self.method == 'POST':
            self.response = self.post(self.url, body)

        return True if self.response else False

    def get_response(self):
        if self.response is not None:
            try:
                return json.loads(self.response.text)
            except json.decoder.JSONDecodeError:
                return self.response.text

        return {}

    def print_response(self):
        print(json.dumps(self.get_response(), indent=4, sort_keys=True))


class MouserBaseRequest(MouserAPIRequest):
    """ Mouser Base Request """

    name = ''
    allowed_methods = ['GET', 'POST']
    operation = None
    operations = {}

    def __init__(self, operation, file_keys=None, *args):
        ''' Init '''

        if operation not in self.operations:
            print(f'[{self.name}]\tInvalid Operation')
            print('-' * 10)
            
            valid_operations = [operation for operation, values in self.operations.items() if values[0] and values[1]]
            if valid_operations:
                print('Valid operations:')
                for operation in valid_operations:
                    print(f'- {operation}')
            return
            
        self.operation = operation
        (method, url) = self.operations.get(self.operation, ('', ''))

        if not url or not method or method not in self.allowed_methods:
            print(f'[{self.name}]\tOperation "{operation}" Not Yet Supported')
            return

        super().__init__(url, method, file_keys, *args)

    def export_csv(self, file_path: str, data: dict):
        ''' Export dictionary data to CSV '''

        with open(file_path, 'w') as csvfile:
            csvwriter = csv.writer(csvfile)

            for row in data:
                csvwriter.writerow(row)
